@@ -4,7 +4,6 @@ import (
 	"bytes"
 	"context"
 	"encoding/hex"
-	"encoding/json"
 	"errors"
 	"io"
 	"os"
@@ -20,8 +19,11 @@ import (
 	imgspecv1 "github.com/opencontainers/image-spec/specs-go/v1"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+	"golang.org/x/exp/slices"
 )
 
+const commonFixtureConfigDigest = "sha256:9ca4bda0a6b3727a6ffcc43e981cad0f24e2ec79d338f6ba325b4dfd0756fb8f"
+
 func manifestSchema2FromFixture(t *testing.T, src types.ImageSource, fixture string, mustFail bool) genericManifest {
 	manifest, err := os.ReadFile(filepath.Join("fixtures", fixture))
 	require.NoError(t, err)
@@ -39,7 +41,7 @@ func manifestSchema2FromComponentsLikeFixture(configBlob []byte) genericManifest
 	return manifestSchema2FromComponents(manifest.Schema2Descriptor{
 		MediaType: "application/octet-stream",
 		Size:      5940,
-		Digest:    "sha256:9ca4bda0a6b3727a6ffcc43e981cad0f24e2ec79d338f6ba325b4dfd0756fb8f",
+		Digest:    commonFixtureConfigDigest,
 	}, nil, configBlob, []manifest.Schema2Descriptor{
 		{
 			MediaType: "application/vnd.docker.image.rootfs.diff.tar.gzip",
@@ -91,20 +93,10 @@ func TestManifestSchema2Serialize(t *testing.T) {
 	} {
 		serialized, err := m.serialize()
 		require.NoError(t, err)
-		var contents map[string]interface{}
-		err = json.Unmarshal(serialized, &contents)
-		require.NoError(t, err)
-
-		original, err := os.ReadFile("fixtures/schema2.json")
-		require.NoError(t, err)
-		var originalContents map[string]interface{}
-		err = json.Unmarshal(original, &originalContents)
-		require.NoError(t, err)
-
 		// We would ideally like to compare “serialized” with some transformation of
-		// “original”, but the ordering of fields in JSON maps is undefined, so this is
+		// the original fixture, but the ordering of fields in JSON maps is undefined, so this is
 		// easier.
-		assert.Equal(t, originalContents, contents)
+		assertJSONEqualsFixture(t, serialized, "schema2.json")
 	}
 }
 
@@ -124,7 +116,7 @@ func TestManifestSchema2ConfigInfo(t *testing.T) {
 	} {
 		assert.Equal(t, types.BlobInfo{
 			Size:      5940,
-			Digest:    "sha256:9ca4bda0a6b3727a6ffcc43e981cad0f24e2ec79d338f6ba325b4dfd0756fb8f",
+			Digest:    commonFixtureConfigDigest,
 			MediaType: "application/octet-stream",
 		}, m.ConfigInfo())
 	}
@@ -133,11 +125,12 @@ func TestManifestSchema2ConfigInfo(t *testing.T) {
 // configBlobImageSource allows testing various GetBlob behaviors in .ConfigBlob()
 type configBlobImageSource struct {
 	mocks.ForbiddenImageSource // We inherit almost all of the methods, which just panic()
+	expectedDigest             digest.Digest
 	f                          func() (io.ReadCloser, int64, error)
 }
 
 func (f configBlobImageSource) GetBlob(ctx context.Context, info types.BlobInfo, _ types.BlobInfoCache) (io.ReadCloser, int64, error) {
-	if info.Digest.String() != "sha256:9ca4bda0a6b3727a6ffcc43e981cad0f24e2ec79d338f6ba325b4dfd0756fb8f" {
+	if info.Digest != f.expectedDigest {
 		panic("Unexpected digest in GetBlob")
 	}
 	return f.f()
@@ -173,7 +166,10 @@ func TestManifestSchema2ConfigBlob(t *testing.T) {
 	} {
 		var src types.ImageSource
 		if c.cbISfn != nil {
-			src = configBlobImageSource{f: c.cbISfn}
+			src = configBlobImageSource{
+				expectedDigest: commonFixtureConfigDigest,
+				f:              c.cbISfn,
+			}
 		} else {
 			src = nil
 		}
@@ -360,6 +356,7 @@ func newSchema2ImageSource(t *testing.T, dockerRef string) *schema2ImageSource {
 
 	return &schema2ImageSource{
 		configBlobImageSource: configBlobImageSource{
+			expectedDigest: commonFixtureConfigDigest,
 			f: func() (io.ReadCloser, int64, error) {
 				return io.NopCloser(bytes.NewReader(realConfigJSON)), int64(len(realConfigJSON)), nil
 			},
@@ -438,11 +435,11 @@ func modifiedLayerInfos(t *testing.T, input []types.BlobInfo) ([]types.BlobInfo,
 		require.NoError(t, err)
 		oldDigest[len(oldDigest)-1] ^= 1
 		b2.Digest = digest.NewDigestFromEncoded(b2.Digest.Algorithm(), hex.EncodeToString(oldDigest))
-		b2.Size = b2.Size ^ 1
+		b2.Size ^= 1
 		modified = append(modified, b2)
 	}
 
-	copy := append([]types.BlobInfo{}, modified...)
+	copy := slices.Clone(modified)
 	return modified, copy
 }
 
@@ -451,7 +448,7 @@ func TestManifestSchema2UpdatedImage(t *testing.T) {
 	original := manifestSchema2FromFixture(t, originalSrc, "schema2.json", false)
 
 	// LayerInfos:
-	layerInfos := append(original.LayerInfos()[1:], original.LayerInfos()[0])
+	layerInfos := append(slices.Clone(original.LayerInfos()[1:]), original.LayerInfos()[0])
 	res, err := original.UpdatedImage(context.Background(), types.ManifestUpdateOptions{
 		LayerInfos: layerInfos,
 	})
@@ -519,15 +516,51 @@ func TestConvertToManifestOCI(t *testing.T) {
 	convertedJSON, mt, err := res.Manifest(context.Background())
 	require.NoError(t, err)
 	assert.Equal(t, imgspecv1.MediaTypeImageManifest, mt)
+	assertJSONEqualsFixture(t, convertedJSON, "schema2-to-oci1.json")
 
-	byHandJSON, err := os.ReadFile("fixtures/schema2-to-oci1.json")
+	convertedConfig, err := res.ConfigBlob(context.Background())
 	require.NoError(t, err)
-	var converted, byHand map[string]interface{}
-	err = json.Unmarshal(byHandJSON, &byHand)
+	assertJSONEqualsFixture(t, convertedConfig, "schema2-to-oci1-config.json")
+
+	// Conversion to OCI with encryption is possible.
+	res, err = original.UpdatedImage(context.Background(), types.ManifestUpdateOptions{
+		LayerInfos:       layerInfosWithCryptoOperation(original.LayerInfos(), types.Encrypt),
+		ManifestMIMEType: imgspecv1.MediaTypeImageManifest,
+	})
 	require.NoError(t, err)
-	err = json.Unmarshal(convertedJSON, &converted)
+	convertedJSON, mt, err = res.Manifest(context.Background())
 	require.NoError(t, err)
-	assert.Equal(t, byHand, converted)
+	assert.Equal(t, imgspecv1.MediaTypeImageManifest, mt)
+	// Layers have been updated as expected
+	ociManifest, err := manifestOCI1FromManifest(originalSrc, convertedJSON)
+	require.NoError(t, err)
+	assert.Equal(t, []types.BlobInfo{
+		{
+			Digest:    "sha256:6a5a5368e0c2d3e5909184fa28ddfd56072e7ff3ee9a945876f7eee5896ef5bb",
+			Size:      51354364,
+			MediaType: "application/vnd.oci.image.layer.v1.tar+gzip+encrypted",
+		},
+		{
+			Digest:    "sha256:1bbf5d58d24c47512e234a5623474acf65ae00d4d1414272a893204f44cc680c",
+			Size:      150,
+			MediaType: "application/vnd.oci.image.layer.v1.tar+gzip+encrypted",
+		},
+		{
+			Digest:    "sha256:8f5dc8a4b12c307ac84de90cdd9a7f3915d1be04c9388868ca118831099c67a9",
+			Size:      11739507,
+			MediaType: "application/vnd.oci.image.layer.v1.tar+gzip+encrypted",
+		},
+		{
+			Digest:    "sha256:bbd6b22eb11afce63cc76f6bc41042d99f10d6024c96b655dafba930b8d25909",
+			Size:      8841833,
+			MediaType: "application/vnd.oci.image.layer.v1.tar+gzip+encrypted",
+		},
+		{
+			Digest:    "sha256:960e52ecf8200cbd84e70eb2ad8678f4367e50d14357021872c10fa3fc5935fa",
+			Size:      291,
+			MediaType: "application/vnd.oci.image.layer.v1.tar+gzip+encrypted",
+		},
+	}, ociManifest.LayerInfos())
 }
 
 func TestConvertToManifestOCIAllMediaTypes(t *testing.T) {
@@ -540,15 +573,11 @@ func TestConvertToManifestOCIAllMediaTypes(t *testing.T) {
 	convertedJSON, mt, err := res.Manifest(context.Background())
 	require.NoError(t, err)
 	assert.Equal(t, imgspecv1.MediaTypeImageManifest, mt)
+	assertJSONEqualsFixture(t, convertedJSON, "schema2-all-media-types-to-oci1.json")
 
-	byHandJSON, err := os.ReadFile("fixtures/schema2-all-media-types-to-oci1.json")
+	convertedConfig, err := res.ConfigBlob(context.Background())
 	require.NoError(t, err)
-	var converted, byHand map[string]interface{}
-	err = json.Unmarshal(byHandJSON, &byHand)
-	require.NoError(t, err)
-	err = json.Unmarshal(convertedJSON, &converted)
-	require.NoError(t, err)
-	assert.Equal(t, byHand, converted)
+	assertJSONEqualsFixture(t, convertedConfig, "schema2-to-oci1-config.json")
 }
 
 func TestConvertToOCIWithInvalidMIMEType(t *testing.T) {
@@ -572,19 +601,10 @@ func TestConvertToManifestSchema1(t *testing.T) {
 	require.NoError(t, err)
 	assert.Equal(t, manifest.DockerV2Schema1SignedMediaType, mt)
 
-	// byDockerJSON is the result of asking the Docker Hub for a schema1 manifest,
+	// schema2-to-schema1-by-docker.json is the result of asking the Docker Hub for a schema1 manifest,
 	// except that we have replaced "name" to verify that the ref from
 	// memoryDest, not from originalSrc, is used.
-	byDockerJSON, err := os.ReadFile("fixtures/schema2-to-schema1-by-docker.json")
-	require.NoError(t, err)
-	var converted, byDocker map[string]interface{}
-	err = json.Unmarshal(byDockerJSON, &byDocker)
-	require.NoError(t, err)
-	err = json.Unmarshal(convertedJSON, &converted)
-	require.NoError(t, err)
-	delete(byDocker, "signatures")
-	delete(converted, "signatures")
-	assert.Equal(t, byDocker, converted)
+	assertJSONEqualsFixture(t, convertedJSON, "schema2-to-schema1-by-docker.json", "signatures")
 
 	assert.Equal(t, GzippedEmptyLayer, memoryDest.storedBlobs[GzippedEmptyLayerDigest])
 
@@ -624,6 +644,16 @@ func TestConvertToManifestSchema1(t *testing.T) {
 		{Digest: GzippedEmptyLayerDigest, Size: -1},
 	}, s1Manifest.LayerInfos())
 
+	// Conversion to schema1 with encryption fails
+	_, err = original.UpdatedImage(context.Background(), types.ManifestUpdateOptions{
+		LayerInfos:       layerInfosWithCryptoOperation(original.LayerInfos(), types.Encrypt),
+		ManifestMIMEType: manifest.DockerV2Schema1SignedMediaType,
+		InformationOnly: types.ManifestUpdateInformation{
+			Destination: memoryDest,
+		},
+	})
+	assert.Error(t, err)
+
 	// FIXME? Test also the various failure cases, if only to see that we don't crash?
 }
 
