@@ -16,6 +16,7 @@ const (
 	digestUncompressed        = digest.Digest("sha256:2222222222222222222222222222222222222222222222222222222222222222")
 	digestCompressedA         = digest.Digest("sha256:3333333333333333333333333333333333333333333333333333333333333333")
 	digestCompressedB         = digest.Digest("sha256:4444444444444444444444444444444444444444444444444444444444444444")
+	digestUncompressedC       = digest.Digest("sha256:7777777777777777777777777777777777777777777777777777777777777777")
 	digestCompressedUnrelated = digest.Digest("sha256:5555555555555555555555555555555555555555555555555555555555555555")
 	compressorNameU           = "compressorName/U"
 	compressorNameA           = "compressorName/A"
@@ -26,7 +27,7 @@ const (
 // GenericCache runs an implementation-independent set of tests, given a
 // newTestCache, which can be called repeatedly and always returns a fresh cache instance
 func GenericCache(t *testing.T, newTestCache func(t *testing.T) blobinfocache.BlobInfoCache2) {
-	for _, s := range []struct {
+	subs := []struct {
 		name string
 		fn   func(t *testing.T, cache blobinfocache.BlobInfoCache2)
 	}{
@@ -35,12 +36,25 @@ func GenericCache(t *testing.T, newTestCache func(t *testing.T) blobinfocache.Bl
 		{"RecordKnownLocations", testGenericRecordKnownLocations},
 		{"CandidateLocations", testGenericCandidateLocations},
 		{"CandidateLocations2", testGenericCandidateLocations2},
-	} {
-		t.Run(s.name, func(t *testing.T) {
+	}
+
+	// Without Open()/Close()
+	for _, s := range subs {
+		t.Run("no Open: "+s.name, func(t *testing.T) {
 			cache := newTestCache(t)
 			s.fn(t, cache)
 		})
 	}
+
+	// With Open()/Close()
+	for _, s := range subs {
+		t.Run("with Open: "+s.name, func(t *testing.T) {
+			cache := newTestCache(t)
+			cache.Open()
+			defer cache.Close()
+			s.fn(t, cache)
+		})
+	}
 }
 
 func testGenericUncompressedDigest(t *testing.T, cache blobinfocache.BlobInfoCache2) {
@@ -205,6 +219,27 @@ func testGenericCandidateLocations2(t *testing.T, cache blobinfocache.BlobInfoCa
 		assert.Equal(t, []blobinfocache.BICReplacementCandidate2{}, cache.CandidateLocations2(transport, scope, digestUnknown, false))
 		assert.Equal(t, []blobinfocache.BICReplacementCandidate2{}, cache.CandidateLocations2(transport, scope, digestUnknown, true))
 
+		// If a record exists with compression without Location then
+		// then return a record without location and with `UnknownLocation: true`
+		cache.RecordDigestCompressorName(digestUncompressedC, "somecompression")
+		assert.Equal(t, []blobinfocache.BICReplacementCandidate2{
+			{
+				Digest:          digestUncompressedC,
+				CompressorName:  "somecompression",
+				UnknownLocation: true,
+				Location:        types.BICLocationReference{Opaque: ""},
+			}}, cache.CandidateLocations2(transport, scope, digestUncompressedC, true))
+		// When another entry with scope and Location is set then it should be returned as it has higher
+		// priority.
+		cache.RecordKnownLocation(transport, scope, digestUncompressedC, types.BICLocationReference{Opaque: "somelocation"})
+		assert.Equal(t, []blobinfocache.BICReplacementCandidate2{
+			{
+				Digest:          digestUncompressedC,
+				CompressorName:  "somecompression",
+				UnknownLocation: false,
+				Location:        types.BICLocationReference{Opaque: "somelocation"},
+			}}, cache.CandidateLocations2(transport, scope, digestUncompressedC, true))
+
 		// Record "2" entries before "1" entries; then results should sort "1" (more recent) before "2" (older)
 		for _, suffix := range []string{"2", "1"} {
 			for _, e := range digestNameSet {
@@ -212,7 +247,9 @@ func testGenericCandidateLocations2(t *testing.T, cache blobinfocache.BlobInfoCa
 			}
 		}
 
-		// Clear any "known" compression values, except on the first loop where they've never been set
+		// Clear any "known" compression values, except on the first loop where they've never been set.
+		// This probably triggers “Compressor for blob with digest … previously recorded as …, now unknown” warnings here, for test purposes;
+		// that shouldn’t happen in real-world usage.
 		if scopeIndex != 0 {
 			for _, e := range digestNameSet {
 				cache.RecordDigestCompressorName(e.d, blobinfocache.UnknownCompression)
